/// Render Module
///
/// all of the code pertaining to Rendering will be written here
///
/// this also includes all Bevy Components and Resources relating to rendering

pub mod shader;
pub mod animation;


use core::slice;

use bevy_ecs::prelude::*;
use glow::HasContext;

/// OpenGL context Resource
#[derive(Resource)]
pub struct GL(pub std::sync::Arc<glow::Context>);

/// Default Vertex Array Object, Every 2d Square object with a texture would only need this
#[derive(Resource)]
pub struct DefaultVAO(pub glow::NativeVertexArray);

/// Default Shader, this will be much more likely to be different
///
/// for all systems The Default Shader is assumed to be the animation shader located at data/shaders/AnimationShader
#[derive(Resource)]
pub struct DefaultShader(pub shader::Shader);

// NOTE systems start here

// a square with texture coordinates
//
// in a line
const VERTICES: [f32; 20] = [
//  X       Y       Z       xTex   yTex
    0.5,    0.5,    0.0,    1.0,    1.0,
    0.5,    -0.5,   0.0,    1.0,    0.0,
    -0.5,   -0.5,   0.0,    0.0,    0.0,
    -0.5,   0.5,    0.0,    0.0,    1.0
];

// indices for the
const INDICES: [u32; 6] = [
    0,  1,  3,
    1,  2,  3
];

pub fn initialize_defaults(mut commands: Commands, gl: Res<GL>)
{
    // create vertex array
    let vao = unsafe {
        Some(gl.0.create_vertex_array().unwrap())
    };

    unsafe {
        // create buffers
        let vbo = Some(gl.0.create_buffer().unwrap());
        let ebo = Some(gl.0.create_buffer().unwrap());


        gl.0.bind_buffer(glow::ARRAY_BUFFER, vbo);

        // pass buffers
        let data = core::slice::from_raw_parts(
            VERTICES.as_ptr() as *const u8,
            VERTICES.len() * core::mem::size_of::<f32>()
        );

        gl.0.named_buffer_data_u8_slice(vbo.unwrap(), data, glow::STATIC_DRAW);

        gl.0.bind_buffer(glow::ELEMENT_ARRAY_BUFFER, ebo);

        let data = slice::from_raw_parts(
            INDICES.as_ptr() as *const u8,
            INDICES.len() * core::mem::size_of::<u32>(),
        );

        gl.0.named_buffer_data_u8_slice(ebo.unwrap(), data, glow::STATIC_DRAW);

        let size = core::mem::size_of::<f32>() as i32;

        // Vertex Info
        gl.0.enable_vertex_attrib_array(0);
        gl.0.vertex_attrib_pointer_f32(0, 3, glow::FLOAT, false, size*5, 0);

        // Texture Coordinates
        gl.0.enable_vertex_attrib_array(2);
        gl.0.vertex_attrib_pointer_f32(1, 2, glow::FLOAT, false, size*5, size*3);

        // unbind buffers and arrays
        gl.0.bind_buffer(glow::ARRAY_BUFFER, None);

        gl.0.bind_vertex_array(None);
    }

    // insert Default Vao
    commands.insert_resource(DefaultVAO(vao.unwrap()));

    let shader = shader::Shader::new("AnimationShader", &gl.0);

    commands.insert_resource(DefaultShader(shader));
}
